#!/usr/bin/env python
import sliderepl
sliderepl.SADeck.run()

### slide:: 1 Setup

from sqlalchemy import *

meta = MetaData(create_engine('sqlite://'))

users = Table('users', meta,
    Column('id', Integer, primary_key=True),
    Column('name', String(50))
)

addresses = Table('addresses', meta,
    Column('id', Integer, primary_key=True),
    Column('email', String(50)),
    Column('user_id', Integer, ForeignKey('users.id'))
)

meta.create_all()

ins = users.insert()

ins.execute([
   {'name':'jack'},
   {'name':'ed'},
   {'name':'wendy'},
   {'name':'mary'}
])

ins = addresses.insert()
ins.execute([
   {'user_id':1, 'email':'jack@jack.com'},
   {'user_id':2, 'email':'ed@yahoo.com'},
   {'user_id':2, 'email':'ed@msn.com'},
   {'user_id':3, 'email':'wendy@nyt.com'},
])

from sqlalchemy.orm import *
Session = scoped_session(sessionmaker(transactional=True,autoflush=False))

class Base(object):
    def __init__(self, **kwargs):
        for key, value in kwargs.iteritems():
            setattr(self, key, value)
    def __repr__(self):
        cls = self.__class__
        return "%s(%s)" % (
            cls.__name__,
            ", ".join(["%s=%r" % (c, getattr(self, c)) for c in cls.c.keys()])
        )
    query = Session.query_property()

### slide:: 2 Class / Mapper Setup

class User(Base):
    pass

class Address(Base):
    pass

mapper(User, users, properties={
   'addresses':relation(Address, backref='user', cascade='all, delete-orphan')
})

mapper(Address, addresses)


### slide::  3 Embedding Expressions

u1 = User.query.get(2)
u1.name = User.name + ' smith'

### slide:: 4
Session.flush()

### slide:: 5 Merge

sess = create_session()
u1 = sess.query(User).get(2)

### slide:: 6
u1.addresses.append(Address(email='ed@gmail.com'))

### slide:: 7
u1.name = 'ed'
u1 = Session.merge(u1)

### slide:: 8
Session.flush()

### slide:: 9 Merge is Not "Get or Create"

u1 = User(id=2)
u2 = Session.merge(u1)
u2 is User.query.get(2)

Session.clear()

### slide:: 10 Attribute Expiry / Refreshery

u1 = User.query.get(2)

### slide:: 11
u1.name = 'foobarfoobar'
u1.addresses = [Address(email='lala')]

### slide:: 12
Session.expire(u1)

print u1.name, u1.addresses

### slide:: 13 Attribute Expiry / Refreshery

u1 = User.query.get(2)

u1.name = 'foobarfoobar'
u1.addresses = [Address(email='lala')]

### slide:: 14
Session.expire(u1, ['name'])

print u1.name, u1.addresses

### slide:: 15 Session Binding
Session.remove()  # remove current session
conn = meta.bind.connect() # connect
Session(bind=conn) # create session with bind

Session.save(User(name='fred', addresses=[Address(email='fred@gmail.com')]))
Session.commit()

Session.remove()

### slide:: 16 Joining External Transactions

Session.remove()

conn = meta.bind.connect()
trans = conn.begin()

Session(bind=conn) 
u = User.query.filter_by(name='fred').one()
u.addresses.append(Address(email='xyz@python.org'))
Session.commit()
Session.remove()

trans.commit()

### slide:: end
